@@ -4,6 +4,7 @@
  *
  * Authors: Marcos Morais <morais@dee.ufcg.edu.br>
  *          Kyller Gorgonio <kyllercg@gmail.com>
+ *          Andre Rodrigues <andre.rodrigues@signove.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,6 +43,7 @@ static void nm_wlan_setup_defaults (NmWlanSetup *, gboolean);
 static void nm_wlan_setup_create_widget (NmWlanSetup *);
 static void nm_wlan_setup_enable_cb (GtkToggleButton *, gpointer);
 static void nm_wlan_setup_enable_options (NmWlanSetup *, gboolean);
+static int nm_wlan_setup_enable_dhcpd(NmWlanSetup *, gboolean);
 
 typedef struct _NmWlanSetupPrivate NmWlanSetupPrivate;
 
@@ -53,11 +55,15 @@ struct _NmWlanSetupPrivate
     GtkWidget *gw_label;
     GtkWidget *mask_label;
     GtkWidget *essid_label;
+    GtkWidget *dhcpd_label;
+    GtkWidget *wep_label;
 
     GtkWidget *ip_entry;
     GtkWidget *gw_entry;
     GtkWidget *mask_entry;
     GtkWidget *essid_entry;
+    GtkWidget *dhcpd_entry;
+    GtkWidget *wep_entry;
 
     NmWlanUtils *wlan_utils;
     
@@ -152,41 +158,55 @@ nm_wlan_setup_create_widget(NmWlanSetup *self)
 
     priv = NM_WLAN_SETUP_GET_PRIVATE (self);
 
-    table = gtk_table_new (5, 2, TRUE);
+    table = gtk_table_new (7, 2, TRUE);
 
-    priv->checkbutton = gtk_check_button_new_with_label ("enable");
+    priv->checkbutton = gtk_check_button_new_with_label (_("enable"));
     g_signal_connect (priv->checkbutton, "toggled", G_CALLBACK(nm_wlan_setup_enable_cb),
                       self);
-
+    
+    /* DHCPD server */
+    priv->dhcpd_label = gtk_label_new(_("DHCP Server: "));      
+    gtk_misc_set_alignment(GTK_MISC(priv->dhcpd_label), 1, 0.5);
+    priv->dhcpd_entry = gtk_check_button_new ();
+    
     /* IP address */
-    priv->ip_label = gtk_label_new("IP address: ");
+    priv->ip_label = gtk_label_new(_("IP address: "));
     gtk_misc_set_alignment(GTK_MISC(priv->ip_label), 1, 0.5);
     priv->ip_entry = gtk_entry_new();
 
     /* Gateway */
-    priv->gw_label = gtk_label_new("Gateway: ");
+    priv->gw_label = gtk_label_new(_("Gateway: "));
     gtk_misc_set_alignment(GTK_MISC(priv->gw_label), 1, 0.5);
     priv->gw_entry = gtk_entry_new();
 
     /* Network mask */
-    priv->mask_label = gtk_label_new("Network mask: ");
+    priv->mask_label = gtk_label_new(_("Network mask: "));
     gtk_misc_set_alignment(GTK_MISC(priv->mask_label), 1, 0.5);
     priv->mask_entry = gtk_entry_new();
 
     /* ESSID */
-    priv->essid_label = gtk_label_new("ESSID: ");
+    priv->essid_label = gtk_label_new(_("ESSID: "));
     gtk_misc_set_alignment(GTK_MISC(priv->essid_label), 1, 0.5);
     priv->essid_entry = gtk_entry_new();
+    
+    /* WEP KEY */
+    priv->wep_label = gtk_label_new(_("WEP Key: "));
+    gtk_misc_set_alignment(GTK_MISC(priv->wep_label), 1, 0.5);
+    priv->wep_entry = gtk_entry_new();
 
     gtk_table_attach_defaults(GTK_TABLE(table), priv->checkbutton, 0, 1, 0, 1);
-    gtk_table_attach_defaults(GTK_TABLE(table), priv->ip_label, 0, 1, 1, 2);
-    gtk_table_attach_defaults(GTK_TABLE(table), priv->ip_entry, 1, 2, 1, 2);
-    gtk_table_attach_defaults(GTK_TABLE(table), priv->gw_label, 0, 1, 2, 3);
-    gtk_table_attach_defaults(GTK_TABLE(table), priv->gw_entry, 1, 2, 2, 3);
-    gtk_table_attach_defaults(GTK_TABLE(table), priv->mask_label, 0, 1, 3, 4);
-    gtk_table_attach_defaults(GTK_TABLE(table), priv->mask_entry, 1, 2, 3, 4);
-    gtk_table_attach_defaults(GTK_TABLE(table), priv->essid_label, 0, 1, 4, 5);
-    gtk_table_attach_defaults(GTK_TABLE(table), priv->essid_entry, 1, 2, 4, 5);
+    gtk_table_attach_defaults(GTK_TABLE (table), priv->dhcpd_label, 0, 1, 1, 2);
+    gtk_table_attach_defaults(GTK_TABLE (table), priv->dhcpd_entry, 1, 2, 1, 2);
+    gtk_table_attach_defaults(GTK_TABLE(table), priv->ip_label, 0, 1, 2, 3);
+    gtk_table_attach_defaults(GTK_TABLE(table), priv->ip_entry, 1, 2, 2, 3);
+    gtk_table_attach_defaults(GTK_TABLE(table), priv->gw_label, 0, 1, 3, 4);
+    gtk_table_attach_defaults(GTK_TABLE(table), priv->gw_entry, 1, 2, 3, 4);
+    gtk_table_attach_defaults(GTK_TABLE(table), priv->mask_label, 0, 1, 4, 5);
+    gtk_table_attach_defaults(GTK_TABLE(table), priv->mask_entry, 1, 2, 4, 5);
+    gtk_table_attach_defaults(GTK_TABLE(table), priv->essid_label, 0, 1, 5, 6);
+    gtk_table_attach_defaults(GTK_TABLE(table), priv->essid_entry, 1, 2, 5, 6);
+    gtk_table_attach_defaults(GTK_TABLE(table), priv->wep_label, 0, 1, 6, 7);
+    gtk_table_attach_defaults(GTK_TABLE(table), priv->wep_entry, 1, 2, 6, 7);
 
     GtkWidget *scroll;
     scroll = gtk_scrolled_window_new (NULL, NULL);
@@ -199,6 +219,7 @@ nm_wlan_setup_create_widget(NmWlanSetup *self)
     gtk_container_add(GTK_CONTAINER(self), scroll);
 
     nm_wlan_setup_enable_options (self, FALSE);
+    gtk_widget_show_all(GTK_WIDGET(self));
 }
 
 static void
@@ -223,18 +244,16 @@ nm_wlan_setup_defaults (NmWlanSetup *self, gboolean pressed)
         gconf_client_set_string(gcClient, NM_GCONF_KEY_WLAN_NAME, NM_WLAN_DEFAULT_NAME , NULL);
         gconf_client_set_string(gcClient, NM_GCONF_KEY_WLAN_SECURITY, NM_WLAN_DEFAULT_SECURITY , NULL);
 
-/*        value = gconf_client_get_string(gcClient, NM_GCONF_KEY_WLAN_IP , NULL);*/
-/*        gtk_entry_set_text(GTK_ENTRY (priv->ip_entry), value == NULL ? NM_WLAN_DEFAULT_IP : value);*/
-/*        gconf_client_set_string(gcClient, NM_GCONF_KEY_WLAN_IP , value == NULL ? NM_WLAN_DEFAULT_IP : value, NULL);*/
+        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->dhcpd_entry),
+                             nm_settings_get_boolean (settings, priv->env,
+                                                     NM_WLAN_SETTINGS_GROUP,
+                                                     NM_SETTINGS_DHCPD));
+
         
         gtk_entry_set_text (GTK_ENTRY (priv->ip_entry),
                              nm_settings_get_value (settings, priv->env,
                                                      NM_WLAN_SETTINGS_GROUP,
                                                      NM_SETTINGS_IP));        
-        
-/*        value = gconf_client_get_string(gcClient, NM_GCONF_KEY_WLAN_GATEWAY, NULL);*/
-/*        gtk_entry_set_text(GTK_ENTRY (priv->gw_entry), value == NULL ? NM_WLAN_DEFAULT_GATEWAY : value);*/
-/*        gconf_client_set_string(gcClient, NM_GCONF_KEY_WLAN_GATEWAY, value == NULL ? NM_WLAN_DEFAULT_GATEWAY : value, NULL);*/
 
         gtk_entry_set_text (GTK_ENTRY (priv->gw_entry),
                              nm_settings_get_value (settings, priv->env,
@@ -242,49 +261,29 @@ nm_wlan_setup_defaults (NmWlanSetup *self, gboolean pressed)
                                                      NM_SETTINGS_GATEWAY));
 
 
-/*        value = gconf_client_get_string(gcClient, NM_GCONF_KEY_WLAN_MASK, NULL);*/
-/*        gtk_entry_set_text(GTK_ENTRY (priv->mask_entry), value == NULL ? NM_WLAN_DEFAULT_MASK : value);*/
-/*        gconf_client_set_string(gcClient, NM_GCONF_KEY_WLAN_MASK, value == NULL ? NM_WLAN_DEFAULT_MASK : value, NULL);*/
 
         gtk_entry_set_text (GTK_ENTRY (priv->mask_entry),
                              nm_settings_get_value (settings, priv->env,
                                                      NM_WLAN_SETTINGS_GROUP,
                                                      NM_SETTINGS_MASK));
 
-/*        essid_list = gconf_client_get_list (gcClient, NM_GCONF_KEY_WLAN_ESSID, GCONF_VALUE_INT, NULL);*/
-
-/*        if (g_slist_length(essid_list) > 0) {*/
-
-/*            the ESSID value is stored as a list of integers in gconf */
-/*            value = g_malloc(50);*/
-/*            for (l = essid_list, i=0; l != NULL && i < 50; l = l->next, i++)*/
-/*                value[i] = (gchar)(GPOINTER_TO_INT(l->data));*/
-/*            value[i]='\0';*/
-
-/*            gtk_entry_set_text(GTK_ENTRY (priv->essid_entry), value);*/
-
-/*            g_free(value);*/
-/*        } else {*/
-
-/*            gtk_entry_set_text(GTK_ENTRY (priv->essid_entry), NM_WLAN_DEFAULT_ESSID);*/
-/*            value = g_strdup(NM_WLAN_DEFAULT_ESSID);*/
-/*             the ESSID value is stored as a list of integers in gconf */
-/*            for (i = 0; value[i] && i < 50; i++)*/
-/*                essid_list = g_slist_append (essid_list, GINT_TO_POINTER( (gint) (value[i]) ));*/
-
-/*            gconf_client_set_list(gcClient, NM_GCONF_KEY_WLAN_ESSID, GCONF_VALUE_INT, essid_list, NULL);*/
-/*        }*/
         gtk_entry_set_text (GTK_ENTRY (priv->essid_entry),
                              nm_settings_get_value (settings, priv->env,
                                                      NM_WLAN_SETTINGS_GROUP,
                                                      NM_WLAN_SETTINGS_ESSID ));
 
+        gtk_entry_set_text (GTK_ENTRY (priv->wep_entry),
+                             nm_settings_get_value (settings, priv->env,
+                                                     NM_WLAN_SETTINGS_GROUP,
+                                                     NM_WLAN_SETTINGS_WEPKEY ));
+
     } else {
 
         gtk_entry_set_text(GTK_ENTRY (priv->ip_entry), "");
         gtk_entry_set_text(GTK_ENTRY (priv->gw_entry), "");
         gtk_entry_set_text(GTK_ENTRY (priv->mask_entry), "");
         gtk_entry_set_text(GTK_ENTRY (priv->essid_entry), "");
+        gtk_entry_set_text(GTK_ENTRY (priv->wep_entry), "");
     }
 }
 
@@ -310,6 +309,8 @@ nm_wlan_setup_enable_options (NmWlanSetup *self, gboolean enable)
     NmWlanSetupPrivate *priv;
     priv = NM_WLAN_SETUP_GET_PRIVATE (self);
 
+    gtk_widget_set_sensitive (priv->dhcpd_label, enable);
+    gtk_widget_set_sensitive (priv->dhcpd_entry, enable);
     gtk_widget_set_sensitive (priv->ip_label, enable);
     gtk_widget_set_sensitive (priv->ip_entry, enable);
     gtk_widget_set_sensitive (priv->gw_label, enable);
@@ -318,11 +319,16 @@ nm_wlan_setup_enable_options (NmWlanSetup *self, gboolean enable)
     gtk_widget_set_sensitive (priv->mask_entry, enable);
     gtk_widget_set_sensitive (priv->essid_label, enable);
     gtk_widget_set_sensitive (priv->essid_entry, enable);
+    gtk_widget_set_sensitive (priv->wep_label, enable);
+    gtk_widget_set_sensitive (priv->wep_entry, enable);
+    
 }
 
 gboolean
 nm_wlan_setup_apply_changes (NmConnWidget *self)
 {
+    g_debug("[%s] - Applying wlan env", __FUNCTION__);
+    
     NmWlanSetup *wlansetup = NM_WLAN_SETUP (self);
     NmWlanSetupPrivate *priv;
     priv = NM_WLAN_SETUP_GET_PRIVATE (wlansetup);
@@ -339,6 +345,9 @@ nm_wlan_setup_apply_changes (NmConnWidget *self)
 
     gboolean is_active = nm_wlan_utils_is_active ();
 
+    // Disable the dhcp server
+    nm_wlan_setup_enable_dhcpd(wlansetup, FALSE);
+
     if (nm_wlan_setup_is_enabled (wlansetup)) {
         
         if (is_active) {
@@ -351,13 +360,27 @@ nm_wlan_setup_apply_changes (NmConnWidget *self)
         gconf_client_set_string (gcClient, NM_GCONF_KEY_WLAN_HIDDEN, NM_WLAN_DEFAULT_HIDDEN , NULL);
         gconf_client_set_string (gcClient, NM_GCONF_KEY_WLAN_NAME, NM_WLAN_DEFAULT_NAME , NULL);
         gconf_client_set_string (gcClient, NM_GCONF_KEY_WLAN_SECURITY, NM_WLAN_DEFAULT_SECURITY , NULL);
+    
+        
+        value = gtk_entry_get_text (GTK_ENTRY (priv->wep_entry));
+        int wep_size = strlen(value);
+        g_debug("[%s] - wep key: %s , size: %d", __FUNCTION__, value, wep_size);
+        if (wep_size != 0 ) {
+            if (wep_size == 5 || wep_size == 10 || wep_size == 13) {
+                gconf_client_set_string (gcClient, NM_GCONF_KEY_WLAN_SECURITY, "WEP", NULL);
+                gconf_client_set_int (gcClient, NM_GCONF_KEY_WLAN_WEPDEFKEY, 1, NULL);
+                value = gtk_entry_get_text (GTK_ENTRY (priv->wep_entry));
+                gconf_client_set_string (gcClient, NM_GCONF_KEY_WLAN_WEPKEY, value , NULL);
+            }
+            else {
+                hildon_banner_show_information (NULL, NULL, _("The WEP key needs to have 5, 10 or 13 caracters\nIgnoring key"));
+            }
+        }
 
+        
         value = gtk_entry_get_text (GTK_ENTRY (priv->ip_entry));
         gconf_client_set_string (gcClient, NM_GCONF_KEY_WLAN_IP , value, NULL);
 
-        value = gtk_entry_get_text (GTK_ENTRY (priv->gw_entry));
-        gconf_client_set_string (gcClient, NM_GCONF_KEY_WLAN_GATEWAY, value, NULL);
-
         value = gtk_entry_get_text (GTK_ENTRY (priv->mask_entry));
         gconf_client_set_string (gcClient, NM_GCONF_KEY_WLAN_MASK, value, NULL);
 
@@ -372,10 +395,15 @@ nm_wlan_setup_apply_changes (NmConnWidget *self)
 
         if (nm_wlan_utils_activate ()) {
 
-/*            hildon_banner_show_information (NULL, NULL, "WLAN Ad-Hoc network enabled");*/
+            /* Enable dhcpd if checkbox active */
+            if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(priv->dhcpd_entry))) {
+                nm_wlan_setup_enable_dhcpd(wlansetup, TRUE);
+            }
+            
+
         } else {
 
-            hildon_banner_show_information (NULL, NULL, "Error activating WLAN Ad-Hoc network");
+            hildon_banner_show_information (NULL, NULL, _("Error activating WLAN Ad-Hoc network"));
             return FALSE;
         }
     } else if (is_active) {
@@ -393,6 +421,38 @@ nm_wlan_setup_apply_changes (NmConnWidget *self)
     return TRUE;
 }
 
+static int
+nm_wlan_setup_enable_dhcpd(NmWlanSetup *self, gboolean enable)
+{
+    NmWlanSetupPrivate *priv = NM_WLAN_SETUP_GET_PRIVATE (self);
+    
+    system("sudo /usr/sbin/nm-udhcpd stop wlan0");
+    
+    if (enable == TRUE) {
+    	NmSettings *settings = nm_settings_get_context();
+        gboolean share_wlan = nm_settings_get_boolean (settings, priv->env, NM_INTERNET_SETTINGS_GROUP,
+    											       NM_INTERNET_IN_INTERFACE_WLAN);
+        const gchar *ip = gtk_entry_get_text (GTK_ENTRY (priv->ip_entry));
+        const gchar *netmask = gtk_entry_get_text (GTK_ENTRY (priv->mask_entry));
+        gchar command[150];
+
+        sprintf(command, "sudo /usr/sbin/nm-udhcpd start wlan0 %s %s", ip, netmask);
+    	/* if sharing internet through wlan, set dhcp gateway and dns */
+        if (share_wlan == TRUE) {
+            gchar *dns = nm_settings_get_value (settings, priv->env, NM_INTERNET_SETTINGS_GROUP,
+                                                NM_INTERNET_DNS);
+        	gchar internet[50];
+
+        	sprintf(internet, " %s %s", ip, dns);
+        	strcat(command, internet);
+        }
+        g_debug("[%s] - command: %s", __FUNCTION__, command);
+        system(command);
+    }
+    return 1;
+
+}
+
 gboolean 
 nm_wlan_setup_activate_environment (gchar *env)
 {
@@ -426,6 +486,9 @@ nm_wlan_setup_save_changes (NmWlanSetup *self)
                         NM_SETTINGS_ENABLE,  nm_wlan_setup_is_enabled(self));
     
     if (nm_wlan_setup_is_enabled(self)) {
+        nm_settings_set_boolean (settings, priv->env, NM_WLAN_SETTINGS_GROUP,
+                                 NM_SETTINGS_DHCPD, 
+                                 gtk_toggle_button_get_active(priv->dhcpd_entry) );
         nm_settings_set_value (settings, priv->env, NM_WLAN_SETTINGS_GROUP,
                             NM_SETTINGS_IP,
                             gtk_entry_get_text (GTK_ENTRY (priv->ip_entry)));
@@ -438,6 +501,9 @@ nm_wlan_setup_save_changes (NmWlanSetup *self)
         nm_settings_set_value (settings, priv->env, NM_WLAN_SETTINGS_GROUP,
                             NM_WLAN_SETTINGS_ESSID,
                             gtk_entry_get_text (GTK_ENTRY (priv->essid_entry)));
+        nm_settings_set_value (settings, priv->env, NM_WLAN_SETTINGS_GROUP,
+                            NM_WLAN_SETTINGS_WEPKEY,
+                            gtk_entry_get_text (GTK_ENTRY (priv->wep_entry)));
     }
     
     nm_settings_save(settings);
